#ifndef RES_APP_H
#include <ResApplication.h>
#endif


ResApplication::ResApplication(uint32 signature) : BApplication(signature), res(NULL), fich(NULL) {

}
ResApplication::ResApplication(const char *signature) : BApplication(signature), res(NULL), fich(NULL) {

}
ResApplication::ResApplication(BMessage *data) : BApplication(data), res(NULL), fich(NULL) {

}

ResApplication::~ResApplication() {
	CloseResources();
}
#pragma export on
ResApplication* ResApplication::Instantiate(BMessage *data) {
	if (validate_instantiation(data, "ResApplication"))
		return new ResApplication(data);
	else
		return NULL;
}
#pragma export reset

BArchivable*	ResApplication::LoadArchive(type_code type, int32 id) {
	BArchivable* arch = NULL;
	BMessage trymsg(B_ARCHIVED_OBJECT);
	if (FindMessage(type, id, &trymsg) == B_OK) {
		arch = instantiate_object(&trymsg);
	}
	return arch;
}

BResources*	ResApplication::OpenResources() {
	if (res) return res;
	
	status_t e;
	app_info info;	
	
	if (!fich) {
		GetAppInfo(&info);
		fich = new BFile(&info.ref, B_READ_WRITE);
	}
	res = new BResources(fich);
	if (!res->File().IsReadable()) {	//error
		delete res;
		delete fich;
		res = NULL;
		fich = NULL;
		printf("ERROR - Application resource not readable'n");
	}
	return res;
}

void		ResApplication::CloseResources() {
	if (res) delete res;
	if (fich) delete fich;
	res = NULL;
	fich = NULL;
}
BResources*		ResApplication::Resources(bool load_if_needed) {
	if (!res && load_if_needed)
		return OpenResources();
	else
		return res;
}

status_t		ResApplication::StoreArchive(type_code type, int32 id, BArchivable* obj, bool deep, const char* resname) {
	status_t e;
	BMessage arch(B_ARCHIVED_OBJECT);
	
	if (!obj) {printf("WARNING: nothing to store !\n"); return B_ERROR;}
	e = obj->Archive(&arch, deep);
	if (e!=B_OK) {printf("WARNING: an achive error %d !\n", e); return e;}
	return AddMessage(type, id, &arch, resname);
}
ResApplication* ResApplication::res_app() {
	return cast_as(be_app, ResApplication);
}

const char*		ResApplication::find_string(const char* entry_name) {
	char *str = NULL;
	ResApplication* app = res_app();
	if (app && (app->FindString(entry_name, &str)==B_OK))
		return str;
	else
		return entry_name;
}
const char*		ResApplication::find_string(const char* entry_name, long i) {
	char *str = NULL;
	ResApplication* app = res_app();
	if (app && (app->FindString(entry_name, i, &str)==B_OK))
		return str;
	else
		return entry_name;
}

status_t		ResApplication::LoadStrings(type_code type, int32 id, int32 defid) {
	status_t e;
	string_msg.MakeEmpty();
	e = FindMessage(type, id, &string_msg);
	if (e!=B_OK) e = FindMessage(type, defid, &string_msg);
	return e;
}

status_t		ResApplication::FindString(const char* entry_name, char** putin) {
	return string_msg.FindString(entry_name, putin);
}

status_t		ResApplication::FindString(const char* entry_name, long i, char** putin) {
	return string_msg.FindString(entry_name, i, putin);
}
		
status_t		ResApplication::FindMessage(type_code type, int32 id, BMessage* inmsg) {
	void* data;
	size_t si;
	char* name;
	status_t e = B_ERROR;
	
	// printf("in find message\n");
	if (!inmsg) {printf("No message to fill\n"); return B_ERROR;}
	OpenResources();
	if (!res) {printf("Unable to open resource fork\n"); return B_ERROR;}
	
	if (res->GetResourceInfo(type, id, &name, &si)) {
		data = malloc(si);
		if (data) {
			res->ReadResource(type, id, data, 0, si);
			e = inmsg->Unflatten((char*) data);
			free(data);
		} else e = B_ERROR;
	} else {
		// printf("no resource type %d id %d found !\n", type, id);
		e = B_ERROR;
	}
	return e;
}

status_t		ResApplication::AddMessage(type_code type, int32 id, BMessage* msg, const char* resname) {
	void* data;
	size_t si;
	char* gname = NULL;
	char old_name[B_OS_NAME_LENGTH];
	status_t e;
	
	// printf("store the archive : \n");
	msg->PrintToStream();
	
	if (!msg) {printf("WARNING: nothing to store !\n"); return B_ERROR;}
	OpenResources();
	if (!res) {printf("WARNING: no resource part !\n"); return B_ERROR;}
	si = msg->FlattenedSize();
	data = malloc(si);
	if (!data) {printf("WARNING: malloc error !\n"); return B_ERROR;}
	e = msg->Flatten((char*) data, (uint32) si);
	if (e != B_OK) {free(data); printf("WARNING: msg flatten error\n"); return e;}
	
	if (res->GetResourceInfo(type, id, &gname, &si)) {
		if (gname) strncpy(old_name, gname, B_OS_NAME_LENGTH);
		res->RemoveResource(type, id);
	}
	e = res->AddResource(type, id, data, si, resname ? resname : (gname ? old_name: msg->FindString("_name")));
	free(data);
	return e;
}